@@ -211,6 +211,9 @@ class PysimmodsModule(UpgradeModule):
                     )
                     sn_mva = params["params"].get("sn_mva", p_peak_mw)
                     full_id = self.start_model(model_key, info[0], params)
+                    p_min, p_max, q_min, q_max = get_space(
+                        model.lower(), params, p_peak_mw
+                    )
 
                     eid_mapping[full_id] = {
                         "p_mw": p_peak_mw,
@@ -218,43 +221,21 @@ class PysimmodsModule(UpgradeModule):
                         "type": info[1],
                         "sn_mva": sn_mva,
                     }
+                    if p_min != p_max:
+                        self.actuators.append(
+                            create_actuator(full_id, "p_set_mw", p_min, p_max)
+                        )
+                    if q_min != q_max:
+                        self.actuators.append(
+                            create_actuator(
+                                full_id, "q_set_mvar", q_min, q_max
+                            )
+                        )
                     self.actuators.append(
-                        {
-                            "actuator_id": f"{full_id}.p_set_mw",
-                            "action_space": (
-                                f"Box(low=0, high={p_peak_mw}, shape=(1,),"
-                                "dtype=np.float32)"
-                            ),
-                        }
-                    )
-                    self.actuators.append(
-                        {
-                            "actuator_id": f"{full_id}.q_set_mvar",
-                            "action_space": (
-                                f"Box(low={-sn_mva}, high={sn_mva}, shape=(1,),"
-                                "dtype=np.float32)"
-                            ),
-                        }
-                    )
-                    self.actuators.append(
-                        {
-                            "actuator_id": (f"{full_id}.set_percent"),
-                            "action_space": (
-                                f"Box(low=0, high={high}, shape=(1,), "
-                                "dtype=np.float32)"
-                            ),
-                        }
+                        create_actuator(full_id, "set_percent", 0, high)
                     )
                     for attr in self.models[model][3]:
-                        self.sensors.append(
-                            {
-                                "sensor_id": (f"{full_id}.{attr}"),
-                                "observation_space": (
-                                    "Box(low=-10000, high=10000, shape=(1,), "
-                                    "dtype=np.float32)"
-                                ),
-                            }
-                        )
+                        self.sensors.append(create_sensor(full_id, attr))
 
                     model_ctrs[model] += 1
 
@@ -561,3 +542,40 @@ def get_presets(model, p_peak_mw, **kwargs):
         return biogas_preset(pn_max_kw=p_peak_mw * 1e3)
     else:
         raise ValueError(f"Model {model} is unknown.")
+
+
+def create_actuator(
+    full_id: str, attr: str, vmin: float, vmax: float, dtype: str = "np.double"
+):
+    return {
+        "actuator_id": f"{full_id}.{attr}",
+        "action_space": (f"Box(low={vmin}, high={vmax}, dtype={dtype})"),
+    }
+
+
+def create_sensor(
+    full_id: str,
+    attr: str,
+    vmin: float = -10000,
+    vmax: float = 10000,
+    dtype: str = "np.double",
+):
+    return {
+        "sensor_id": f"{full_id}.{attr}",
+        "observation_space": (f"Box(low={vmin}, high={vmax}, dtype={dtype})"),
+    }
+
+
+def get_space(model, params, p_peak_mw):
+    p_min = q_min = q_max = 0
+    p_max = p_peak_mw
+
+    if model.lower() in ["pv"]:
+        q_min = -params["params"]["sn_mva"]
+        q_max = params["params"]["sn_mva"]
+
+    if model.lower() in ["bat"]:
+        p_min = -params["params"]["p_discharge_max_kw"] * 1e-3
+        p_max = params["params"]["p_charge_max_kw"] * 1e-3
+
+    return p_min, p_max, q_min, q_max
